import matplotlib.pyplot as plt
from MPSPlots.styles import mps as plot_style
from functools import wraps

import matplotlib.pyplot as plt
from MPSPlots.styles import mps as plot_style
from functools import wraps
import numpy as np

def pre_plot(nrows: int = 1, ncols: int = 1, subplot_kw: dict = {}):
    """
    Decorator factory that creates a matplotlib figure with subplots
    before calling the decorated plotting function.

    Parameters
    ----------
    nrows : int
        Number of subplot rows.
    ncols : int
        Number of subplot columns.
    """
    def decorator(function):
        @wraps(function)
        def wrapper(*args,
                    show: bool = True,
                    save_as: str = None,
                    figure_size: tuple = None,
                    tight_layout: bool = True,
                    axes: plt.Axes = None,
                    xscale: str = None,
                    yscale: str = None,
                    xlim: tuple = None,
                    ylim: tuple = None,
                    style: str = plot_style,
                    **kwargs):

            with plt.style.context(style):
                if axes is None:
                    figure, axes = plt.subplots(
                        nrows=nrows,
                        ncols=ncols,
                        figsize=figure_size,
                        squeeze=False,
                        subplot_kw=subplot_kw
                    )
                else:
                    figure = axes.get_figure()

                axes = np.array(axes).flatten()
                if nrows * ncols == 1:
                    axes = axes[0]

                # Call the decorated function
                if args and hasattr(args[0], "__class__"):
                    function(args[0], axes=axes, *args[1:], **kwargs)
                else:
                    function(*args, axes=axes, **kwargs)

                # Apply axis customizations
                all_axes = axes if isinstance(axes, np.ndarray) else [axes]
                for ax in all_axes:
                    if xscale:
                        ax.set_xscale(xscale)
                    if yscale:
                        ax.set_yscale(yscale)
                    if xlim:
                        ax.set_xlim(*xlim)
                    if ylim:
                        ax.set_ylim(*ylim)

                if tight_layout:
                    figure.tight_layout()

                if save_as is not None:
                    figure.savefig(save_as, dpi=300)

                if show:
                    plt.show()

                return figure

        return wrapper
    return decorator



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path


__all__ = [
    'root_path',
    'project_path',
    'doc_path',
    'doc_css_path',
    'style_directory',
    'logo_path',
]

root_path = Path(__file__).parents[0]

project_path = root_path.parents[0]

doc_path = root_path.parents[0].joinpath('docs')

doc_css_path = doc_path.joinpath('source/_static/default.css')

fonts_directory = root_path.joinpath('fonts')

style_directory = root_path.joinpath('styles')

logo_path = doc_path.joinpath('images/logo.png')



if __name__ == '__main__':
    for path_name in __all__:
        path = locals()[path_name]
        print(f"{path_name:20s}: {path}")
        assert path.exists(), f"Path [{path_name}] {path_name} do not exists"



#!/usr/bin/env python
# -*- coding: utf-8 -*-


__all__ = [
    "classic",
    "polytechnique"
]

from matplotlib.colors import LinearSegmentedColormap


class NameSpace:
    """
    A class to dynamically create attributes from keyword arguments.
    """

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


blue_black_red = LinearSegmentedColormap.from_list(
    'blue_black_red',
    (
        # Edit this gradient at https://eltos.github.io/gradient/#0:7A90FF-33.9:0025B3-50:000000-75.8:C7030D-100:FF6E75
        (0.000, (0.478, 0.565, 1.000)),
        (0.339, (0.000, 0.145, 0.702)),
        (0.500, (0.000, 0.000, 0.000)),
        (0.758, (0.780, 0.012, 0.051)),
        (1.000, (1.000, 0.431, 0.459))
    )
)


blue_white_red = LinearSegmentedColormap.from_list(
    'blue_white_red',
    (
        # Edit this gradient at https://eltos.github.io/gradient/#0025B3-7A90FF-FFFFFF-FF6E75-C7030D
        (0.000, (0.000, 0.145, 0.702)),
        (0.250, (0.478, 0.565, 1.000)),
        (0.500, (1.000, 1.000, 1.000)),
        (0.750, (1.000, 0.431, 0.459)),
        (1.000, (0.780, 0.012, 0.051))
    )
)

_red_black_blue = LinearSegmentedColormap.from_list(
    'poly_red_black_blue',
    (
        # Edit this gradient at https://eltos.github.io/gradient/#41AAE6-000000-B91E32
        (0.000, (0.255, 0.667, 0.902)),
        (0.500, (0.000, 0.000, 0.000)),
        (1.000, (0.725, 0.118, 0.196))
    )
)

_red_white_blue = LinearSegmentedColormap.from_list('my_gradient', (
    # Edit this gradient at https://eltos.github.io/gradient/#41AAE6-FFFFFF-B91E32
    (0.000, (0.255, 0.667, 0.902)),
    (0.500, (1.000, 1.000, 1.000)),
    (1.000, (0.725, 0.118, 0.196))))

classic = NameSpace(
    blue_black_red=blue_black_red,
    red_black_blue=blue_black_red,
    blue_white_red=blue_white_red,
    red_white_blue=blue_white_red
)

polytechnique = NameSpace(
    red_black_blue=_red_black_blue,
    blue_black_red=_red_black_blue,
    red_white_blue=_red_white_blue,
    blue_white_red=_red_white_blue
)

# -
